
/** 
 * Written by:
 *		Jun Peng (junpeng@stanford.edu)
 *		Kincho Law (law@stanford.edu)
 * ----------------------------------
 * ReportGen defines the functions for generating engineering report.
 * Data are saved in OWL (Web Ontology Language) file.
 */ 


import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;


public class OWLDataRetrieval extends HttpServlet {

	private OWLMapping theMap = new OWLMapping();


	/* Depends on the user's choice, we will take different action to 
	 * display a portion of the project-related data.
	 */
	public void handleGenReport(OntModel theModel, String category, 
							String project, PrintWriter out)
	{       
		if (category.equals("summary")) 
		{
			theMap.getSummary(theModel, project, out); 
		}
		else if (category.equals("scope")) 
		{
			theMap.getScope(theModel, project, out);
		}
		else if (category.equals("setupoverview")) 
		{
			theMap.getSetupOverview(theModel, project, out);
		} 
		else if (category.equals("infrastructure")) 
		{
			theMap.getInfrastructure(theModel, project, out);
		}
		else if (category.equals("specimen")) 
		{
			theMap.getSpecimen(theModel, project, out);
		} 
		else if (category.equals("loading")) 
		{
			theMap.getLoading(theModel, project, out);
		} 
		else if (category.equals("instrumentation")) 
		{
			theMap.getInstrumentation(theModel, project, out);
		} 
		else if (category.equals("procedures")) 
		{
			theMap.getProcedures(theModel, project, out);
		} 
		else if (category.equals("schedule")) 
		{
			theMap.getSchedule(theModel, project, out);
		} 
		else if (category.equals("implementation")) 
		{
			theMap.getImplementation(theModel, project, out);
		} 
		else if (category.equals("results")) 
		{
			theMap.getResults(theModel, project, out);
		}
		else if (category.equals("dataprocessing")) 
		{
			theMap.getDataProcessing(theModel, project, out);
		} 
		else if (category.equals("predications")) 
		{
			theMap.getPredications(theModel, project, out);
		}
    }
    
    /**
	 * Display the page for user to choose certain action.
	 */
    public void genReport(OntModel theModel, PrintWriter out)
    {
		OWLAccess MyAcc = theMap.getOWLAccess();
        out.println("<p>\n" +
                    "<h3>Engineering Report Generation Facility:</h3>");
        out.println("<form method=\"GET\">");
        
        out.println("<b>Please choose the Project first:</b><br>\n" +
                    "<select name=\"THEPROJECT\">\n" +
                    "\t<option value=>\n");
        
        OntClass theProject = theModel.getOntClass(MyConsts.myBase + "Project");       
        for (Iterator i = theProject.listInstances(); i.hasNext(); ) 
		{
            Individual inst = (Individual) i.next();
            String projName = MyAcc.getShortName(inst.toString());
            out.println("\t<option value=\"" + projName + "\">" + projName);
        }
        
        out.println("</select> <br>\n");
        
        out.println("<b>Choose the category that you want to download:</b><br>\n" +
                    "<select name=\"THECHOICE\">\n" +
                    "\t<option value=\"summary\">1. Summary\n" +
                    "\t<option value=\"scope\">2. Scope and general presentation\n" + 
                    "\t<option value=\"setupoverview\">3. Test setup overview\n" + 
					"\t<option value=\"infrastructure\">3.1. Infrastructure setup\n" + 
                    "\t<option value=\"specimen\">\t3.2. Specimen description\n" +
					"\t<option value=\"loading\">3.3. Loading System\n" + 
					"\t<option value=\"instrumentation\">3.4. Instrumentation\n" +
					"\t<option value=\"procedures\">4. Test procedures\n" + 
					"\t<option value=\"schedule\">4.1. Test schedule\n" +
					"\t<option value=\"implementation\">4.2. Test implementation\n" + 
					"\t<option value=\"results\">5. Test results\n" +
					"\t<option value=\"dataprocessing\">6. Data processing\n" + 
					"\t<option value=\"predictions\">7. Analytical predictions\n" +
					"</select> <p>\n");
        
        out.println("<input type=\"submit\" value=\"Submit\" name=\"ReportChoice\">\n" +
		    "</form><br>" ); 	
    }
    

	public void printHeading(PrintWriter out, String heading)
	{
		out.println("<html>");
		out.println("<body bgcolor=\"white\">");
		out.println("<head><H2>" + heading +  "</H2></head>");
		out.println("<hr>");
	}
   

    public void doGet(HttpServletRequest request,
                      HttpServletResponse response)
        throws IOException, ServletException
    {
        response.setContentType("text/html");
        PrintWriter out = response.getWriter();
        OntModel theModel = theMap.getOntModel();
   
		if (request.getParameter("neesobject") != null) 
		{
            printHeading(out, "The Browsing of NEESMeta Objects:");
			OWLAccess myAcc = theMap.getOWLAccess();
            myAcc.displayAnInstance(theModel, request.getParameter("neesobject"), out);
		}
		else if (request.getParameter("ReportChoice") != null)
		{
            printHeading(out, "Engineering Report Generation Facility:");
            handleGenReport(theModel, request.getParameter("THECHOICE"),
                            request.getParameter("THEPROJECT"), out);
		}
		else 
		{
            printHeading(out, "The Retrieving of NEESMeta Objects:");
            genReport(theModel, out);
        }
                
        out.println("</body></html>");
    }

    public void doPost(HttpServletRequest request,
                      HttpServletResponse response)
        throws IOException, ServletException
    {
        doGet(request, response);
    }

}

