/** 
 * Written by:
 *		Jun Peng (junpeng@stanford.edu)
 *		Kincho Law (law@stanford.edu)
 * ----------------------------------
 * OWLMapping defines the functions for mapping data into related groups.
 * Data are saved in OWL (Web Ontology Language) files.
 */ 

import java.io.*;
import java.text.*;
import java.util.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;


public class OWLMapping
{
	private OWLAccess MyAcc = new OWLAccess();


	public OntModel getOntModel() 
	{
		return MyAcc.getOntModel();
	}
	
	public OWLAccess getOWLAccess()
	{
		return MyAcc;
	}


	public void getSummary(OntModel theModel, String project, PrintWriter out)
	{
		out.println("The following data are related to <b>Summary</b>: <p>");
		MyAcc.displayAnInstance(theModel, project, out);    
	}

	public void getScope(OntModel theModel, String project, PrintWriter out)
	{
		out.println("The following data and files are related to <b>Scope and general presentation</b>: <p>");
		Vector publications = MyAcc.getInstanceProperty(theModel, project, "hasPublications");
		for (int i = 0; i < publications.size(); i++) 
		{
			MyAcc.retrieveFile(theModel, publications.elementAt(i).toString(), out);
		}        
	}

	public void getSetupOverview(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("The following data are related to <b>Test setup overview</b>: <p>");

		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector taskNames = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "name");
			String taskName = MyAcc.getFirstPart( taskNames.elementAt(0).toString() );
			out.println("The <b>Test setup overview</b> for task <b>" + taskName + "</b> is as follows: <p>"); 
			Vector infrastructureSetup = MyAcc.getInstanceProperty(theModel, 
				tasks.elementAt(i).toString(), "hasInfrastructureSetup");

			for (int ii = 0; ii < infrastructureSetup.size(); ii++) 
			{
				MyAcc.displayAnInstance(theModel, (Resource)infrastructureSetup.elementAt(i), out);
			}
		}

	}
	
	public void getInfrastructure(OntModel theModel, String project, PrintWriter out)
	{
		out.println("The following files are related to <b>Infrastructure Setup</b>: <p>");

		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector taskNames = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "name");
			String taskName = MyAcc.getFirstPart( taskNames.elementAt(0).toString() );
			out.println("The <b>Infrastructure setup </b> for task <b>" + taskName + "</b> is as follows: <p>"); 
			Vector infrastructureSetup = MyAcc.getInstanceProperty(theModel, 
				tasks.elementAt(i).toString(), "hasInfrastructureSetup");

			retrieveDrawingDescription(theModel, infrastructureSetup, out);
		}
	}

	public void retrieveDrawingDescription(OntModel theModel, Vector resources, PrintWriter out) 
	{
		for (int ii = 0; ii < resources.size(); ii++) 
		{
			MyAcc.displayAnInstance(theModel, (Resource)resources.elementAt(ii), out);
			out.println("<p>The following are related <b>Description</b> files: <p>");
			Vector descriptions = MyAcc.getInstanceProperty(theModel, resources.elementAt(ii).toString(), "setupDescriptions");
			for (int m = 0; m < descriptions.size(); m++) 
			{
				MyAcc.retrieveFile(theModel, descriptions.elementAt(m).toString(), out);
			}
			out.println("<p>The following are related <b>Drawing</b> files: <p>");
			Vector drawings = MyAcc.getInstanceProperty(theModel, resources.elementAt(ii).toString(), "setupFigures");
			for (int m = 0; m < drawings.size(); m++) 
			{
				MyAcc.retrieveFile(theModel, drawings.elementAt(m).toString(), out);
			}		
		}
	}

	public void getSpecimen(OntModel theModel, String project, PrintWriter out)
	{
		out.println("The following files are related to <b>Specimen Setup</b>: <p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{
				Vector specimenSetup = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasSpecimenSetup");
				for (int k = 0; k < specimenSetup.size(); k++) 
				{
					MyAcc.displayAnInstance(theModel, (Resource)specimenSetup.elementAt(k), out);
					Vector drawings = MyAcc.getInstanceProperty(theModel, specimenSetup.elementAt(k).toString(), "setupFigures");
					out.println("The following files are related to general <b>Specimen Drawings</b>: <p>");
					for (int m = 0; m < drawings.size(); m++) 
					{
						MyAcc.retrieveFile(theModel, drawings.elementAt(m).toString(), out);
					}
                    
					out.println("<p>The following files are related to general <b>Specimen Descriptions</b>: <p>");
					Vector descriptions = MyAcc.getInstanceProperty(theModel, specimenSetup.elementAt(k).toString(), "setupDescriptions");                                    
					for (int m = 0; m < descriptions.size(); m++) 
					{
						MyAcc.retrieveFile(theModel, descriptions.elementAt(m).toString(), out);
					}

					out.println("<p>The following files are related to <b>Specimen Drawings</b>: <p>");
					Vector specimen = MyAcc.getInstanceProperty(theModel, specimenSetup.elementAt(k).toString(), "hasSpecimen");
					for (int m = 0; m < specimen.size(); m++) 
					{
						Vector figs = MyAcc.getInstanceProperty(theModel, specimen.elementAt(m).toString(), "hasFigures");
						for (int nn = 0; nn < figs.size(); nn++) 
						{
							MyAcc.retrieveFile(theModel, figs.elementAt(nn).toString(), out);
						}
					}
				}
			}
		}
	}
    
	
	public void getLoading(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("The following files are related to <b>Loading</b>: <p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{
				Vector waveFormSetup = MyAcc.getInstanceProperty(theModel, 
					eventGroups.elementAt(ii).toString(), "hasWaveFormSetup");
				out.println("The following are related to <b>Loading</b> of EventGroups: <p>");
				retrieveWaveFormSetup(theModel, waveFormSetup, out);

				Vector events = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");
				for (int m = 0; m < events.size(); m++) 
				{
					Vector eventWaveForm = MyAcc.getInstanceProperty(theModel,
						events.elementAt(m).toString(), "hasWaveFormSetup");
					
					out.println("The following are related to <b>Loading</b> of Events: <p>");					
					retrieveWaveFormSetup(theModel, eventWaveForm, out);
					
				}
			}
		}        		

	}
	
	public void retrieveWaveFormSetup(OntModel theModel, Vector waveFormSetup, PrintWriter out)
	{
		out.println("The following files are related to <b>Waveform Setup</b>: <p>");
		for (int i = 0; i < waveFormSetup.size(); i++) 
		{
			MyAcc.displayAnInstance(theModel, (Resource)waveFormSetup.elementAt(i), out);

			Vector waveForms = MyAcc.getInstanceProperty(theModel, waveFormSetup.elementAt(i).toString(), "hasWaveForm");
			for (int ii = 0; ii < waveForms.size(); ii++) 
			{
				out.println("<p>");
				MyAcc.displayAnInstance(theModel, (Resource)waveForms.elementAt(ii), out);
				
				Vector waveRecord = MyAcc.getInstanceProperty(theModel, waveForms.elementAt(ii).toString(), "motionRecrod");

				for (int m = 0; m < waveRecord.size(); m++) 
				{
					MyAcc.retrieveFile(theModel, (waveRecord.elementAt(m)).toString(), out);
				}			
			}
		}
	}

	public void getInstrumentation(OntModel theModel, String project, PrintWriter out)
	{
		out.println("The following files are related to <b>Instrumentation Setup</b>: <p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{
				Vector sensorSetup = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasSensorSetup");
				retrieveDrawingDescription(theModel, sensorSetup, out);
			}
		}        
	}    
    

	public void getProcedures(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("<b>The Project level procedure is as follows:</b>");
		retrieveDateTime(theModel, project, out);

		out.println("<hr><b>The Task level procedure is as follows:</b>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			retrieveDateTime(theModel, tasks.elementAt(i).toString(), out);
		}

		out.println("<hr><b>The EventGroup level procedure is as follows:</b>");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{
				retrieveDateTime(theModel, eventGroups.elementAt(ii).toString(), out);
			}
		}

		out.println("<hr><b>The Event level procedure is as follows:</b>");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{
				Vector events = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");
				for (int k = 0; k < events.size(); k++)
				{
					retrieveDateTime(theModel, events.elementAt(k).toString(), out);
				}
			}
		}
	}

	/**
	 * Retrieve and display the startDateTime & endDateTime of an Activity.
	 */
	public void retrieveDateTime(OntModel theModel, String activity, PrintWriter out)
	{
		Vector activityName = MyAcc.getInstanceProperty(theModel, activity, "name");
		String nameStr = new String();
		String sTimeStr = new String();
		String eTimeStr = new String();

		if (activityName.size() != 0) 
		{
			nameStr = MyAcc.getFirstPart(activityName.elementAt(0).toString());
		}
		Vector startDateTime = MyAcc.getInstanceProperty(theModel, activity, "startDateTime");
		if (startDateTime.size() != 0) 
		{
			sTimeStr = MyAcc.getFirstPart(startDateTime.elementAt(0).toString());
		}
		Vector endDateTime = MyAcc.getInstanceProperty(theModel, activity, "endDateTime");
		if (endDateTime.size() != 0) 
		{
			eTimeStr = MyAcc.getFirstPart(endDateTime.elementAt(0).toString());
		}
		out.println("<p>The schedule for the Activity <b>" + nameStr + "</b> is: <br>");
		out.println("startDateTime: " + sTimeStr);
		out.println(" | endDateTime: " + eTimeStr);

	}


	public void getSchedule(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("The following data are related to <b>Project Schedule</b>: <p>");
		MyAcc.displayAnInstance(theModel, project, out);
		getProcedures(theModel, project, out);
		/*
		 displayAnInstance(theModel, project, out);

		 Vector tasks = getInstanceProperty(theModel, project, "hasTasks");
		 for (int i = 0; i < tasks.size(); i++) 
		 {
		 displayAnInstance(theModel, tasks.elementAt(i).toString(), out);
		 Vector eventGroups = getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");

		 for (int ii = 0; ii < eventGroups.size(); ii++) 
		 {
		 displayAnInstance(theModel, eventGroups.elementAt(ii).toString(), out);
		 Vector events = getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");

		 for (int k = 0; k < events.size(); k++) 
		 {
		 displayAnInstance(theModel, events.elementAt(k).toString(), out);
		 }
		 }
		 }
		 */
	}
	

	public void getImplementation(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("<b>The project level implementation is as follows:</b><p>");
		MyAcc.displayAnInstance(theModel, project, out);

		out.println("<hr><b>The task level implementation is as follows:</b><p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			MyAcc.displayAnInstance(theModel, tasks.elementAt(i).toString(), out);
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{
				MyAcc.displayAnInstance(theModel, eventGroups.elementAt(ii).toString(), out);
				Vector events = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");
				for (int k = 0; k < events.size(); k++) 
				{
					MyAcc.displayAnInstance(theModel, events.elementAt(k).toString(), out);
				}
			}
		}
	}

	/**
	 * Test to see if the type of a particular Object is equal to the input "theType".
	 */
	public boolean propertyIsType(OntModel theModel, Resource oneNode, String theType)
	{
		StmtIterator sIter = oneNode.listProperties();
		while (sIter.hasNext()) 
		{
			Statement stmt = sIter.nextStatement();
			Property predicate = stmt.getPredicate();
			if (predicate.getLocalName().startsWith("type"))
			{
				String rType = MyAcc.getShortName(stmt.getObject().toString());
				if (rType.equals(theType)) 
				{
					return true;
				}
			}
		}
		return false;
	}
	


	public void getResults(OntModel theModel, String project, PrintWriter out)
	{
		out.println("<p>The following files are related to <b>Experimental Results</b>: <p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{		
				Vector events = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");
				for (int k = 0; k < events.size(); k++) 
				{
					boolean isExperimentEvent = propertyIsType(theModel, (Resource)events.elementAt(k), "ExperimentEvent");
					if (isExperimentEvent) 
					{
					
						Vector outputs = MyAcc.getInstanceProperty(theModel, events.elementAt(k).toString(), "hasOutputData");
						for (int m = 0; m < outputs.size(); m++) 
						{
							Vector dataelements = MyAcc.getInstanceProperty(theModel, outputs.elementAt(m).toString(), "hasFiles");
							for (int nn = 0; nn < dataelements.size(); nn++) 
							{
								MyAcc.retrieveFile(theModel, dataelements.elementAt(nn).toString(), out);
							}
						}
					}
				
				}				
			}
		}       
	}    
    

	public void getDataProcessing(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("<p>The following files are related to <b>Data Processing Results</b>: <p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{		
				Vector events = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");
				for (int k = 0; k < events.size(); k++) 
				{
					boolean isProcessedEvent = propertyIsType(theModel, (Resource)events.elementAt(k), "ProcessedEvent");
					if (isProcessedEvent) 
					{					
						Vector outputs = MyAcc.getInstanceProperty(theModel, events.elementAt(k).toString(), "hasOutputData");
						for (int m = 0; m < outputs.size(); m++) 
						{
							Vector dataelements = MyAcc.getInstanceProperty(theModel, outputs.elementAt(m).toString(), "hasFiles");
							for (int nn = 0; nn < dataelements.size(); nn++) 
							{
								MyAcc.retrieveFile(theModel, dataelements.elementAt(nn).toString(), out);
							}
						}
					}
				
				}				
			}
		}       

	}

	/**
	 * Retrieve the information regarding computer simulations and predictions.
	 */
	public void getPredications(OntModel theModel, String project, PrintWriter out) 
	{
		out.println("<p>The following files are related to <b>Simulation and Predication Results</b>: <p>");
		Vector tasks = MyAcc.getInstanceProperty(theModel, project, "hasTasks");
		for (int i = 0; i < tasks.size(); i++) 
		{
			Vector eventGroups = MyAcc.getInstanceProperty(theModel, tasks.elementAt(i).toString(), "hasEventGroups");
			for (int ii = 0; ii < eventGroups.size(); ii++) 
			{		
				Vector events = MyAcc.getInstanceProperty(theModel, eventGroups.elementAt(ii).toString(), "hasEvents");
				for (int k = 0; k < events.size(); k++) 
				{
					boolean isSimulationEvent = propertyIsType(theModel, (Resource)events.elementAt(k), "SimulationEvent");
					if (isSimulationEvent) 
					{	
						// retrieve the inputData
						Vector inputs = MyAcc.getInstanceProperty(theModel, events.elementAt(k).toString(), "hasInputFiles");
						for (int m = 0; m < inputs.size(); m++) 
						{
							Vector dataelements = MyAcc.getInstanceProperty(theModel, inputs.elementAt(m).toString(), "hasFiles");
							for (int nn = 0; nn < dataelements.size(); nn++) 
							{
								MyAcc.retrieveFile(theModel, dataelements.elementAt(nn).toString(), out);
							}
						}
						// retrieve the outputData
						Vector outputs = MyAcc.getInstanceProperty(theModel, events.elementAt(k).toString(), "hasOutputData");
						for (int m = 0; m < outputs.size(); m++) 
						{
							Vector dataelements = MyAcc.getInstanceProperty(theModel, outputs.elementAt(m).toString(), "hasFiles");
							for (int nn = 0; nn < dataelements.size(); nn++) 
							{
								MyAcc.retrieveFile(theModel, dataelements.elementAt(nn).toString(), out);
							}
						}
					}
				
				}				
			}
		}       
	}


}