/** 
 * Written by:
 *		Jun Peng (junpeng@stanford.edu)
 *		Kincho Law (law@stanford.edu)
 * ----------------------------------
 * Generate an engineering report based on the data saved in 
 * either OWL files or MySQL databases.
 */ 

import java.io.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdf.model.*;


public class ReportGen extends HttpServlet 
{
	private OWLMapping owlMap = new OWLMapping();
	private DBMapping dbMap = new DBMapping();


	public void printHeading(PrintWriter out, String heading)
	{
		out.println("<html>");
		out.println("<body bgcolor=\"white\">");
		out.println("<head><H2>" + heading +  "</H2></head>");
	}


	private void genOWLFirstPage(PrintWriter out) 
	{
		OntModel theModel = owlMap.getOntModel();
		OWLAccess MyAcc = owlMap.getOWLAccess();
		out.println("<p>\n" +
			"<h3>For Data Saved as OWL Files:</h3>");
		out.println("<form method=\"GET\">");
        
		out.println("<b>Please choose the Project first:</b><br>\n" +
			"<select name=\"THEPROJECT\">\n" +
			"\t<option value=>\n");
        
		OntClass theProject = theModel.getOntClass(MyConsts.myBase + "Project");       
		for (Iterator i = theProject.listInstances(); i.hasNext(); ) 
		{
			Individual inst = (Individual) i.next();
			String projName = MyAcc.getShortName(inst.toString());
			out.println("\t<option value=\"" + projName + "\">" + projName);
		}
        
		out.println("</select> <br>\n");
        
		out.println("<input type=\"submit\" value=\"Submit\" name=\"OWLChoice\">\n" +
			"</form><br>" );
	}


	private void genDBFirstPage(PrintWriter out)
	{
		out.println("<p>\n" +
			"<h3>For Data Saved in Relational Database:</h3>");
		out.println("<form method=\"GET\">");
        
		out.println("<b>Please choose the Project first:</b><br>\n" +
			"<select name=\"THEPROJECT\">\n" +
			"\t<option value=>\n");

		DBAccess dbAccess = dbMap.getDBAccess();
		dbAccess.initdb(out);

		Vector allProjects = dbAccess.getColumnValues("Project", "title");
		for (int i = 0; i < allProjects.size(); i++) 
		{
			String projName = (String)allProjects.elementAt(i);
			out.println("\t<option value=\"" + projName + "\">" + projName);
		}

		out.println("</select> <br>\n");
        
		out.println("<input type=\"submit\" value=\"Submit\" name=\"DBChoice\">\n" +
			"</form><br>" ); 	
	}

	public void genFirstPage(PrintWriter out)
	{
		out.println("<hr>");
		genOWLFirstPage(out);
		genDBFirstPage(out);
	}


	private void setSessionAtt(HttpServletRequest request, String attribute, String value)
	{
		HttpSession session = request.getSession();
		session.setAttribute(attribute, value);
	}

	private String getSessionAtt(HttpServletRequest request, String attribute)
	{
		HttpSession session = request.getSession();
		String value = (String)session.getAttribute(attribute);
		return value;
	}


	private void genBackButton(PrintWriter out)
	{
		out.print("<form method=\"GET\">\n" +
			"<input type=submit name=\"b_back\" value=\"Back to Project Selection\"><br>\n" +
			"</form>");
		out.println("<hr>");
	}


	private void dealChoice(PrintWriter out, String theProject, String theChoice, String dType)
	{
		if (dType.equals("DB")) 
		{
			DBAccess dbAccess = dbMap.getDBAccess();
			dbAccess.initdb(out);
			handleDBGenReport(theProject, theChoice.toLowerCase(), out);
		} 
		else if (dType.equals("OWL")) 
		{
			OntModel theModel = owlMap.getOntModel();
			handleOWLGenReport(theModel, theChoice.toLowerCase(), theProject, out);
		}
	}


	/* Depends on the user's choice, we will take different action to 
	 * display a portion of the project-related data.
	 */
	public void handleDBGenReport(String project, String category, PrintWriter out)
	{       
		if (category.equals("summary")) 
		{
			dbMap.getSummary(project, out); 
		}
		else if (category.equals("scope")) 
		{
			dbMap.getScope(project, out);
		}
		else if (category.equals("setupoverview")) 
		{
			dbMap.getSetupOverview(project, out);
		} 
		else if (category.equals("infrastructure")) 
		{
			dbMap.getInfrastructure(project, out);
		}
		else if (category.equals("specimen")) 
		{
			dbMap.getSpecimen(project, out);
		} 
		else if (category.equals("loading")) 
		{
			dbMap.getLoading(project, out);
		} 
		else if (category.equals("instrumentation")) 
		{
			dbMap.getInstrumentation(project, out);
		}
		else if (category.equals("schedule")) 
		{
			dbMap.getSchedule(project, out);
		} 
		else if (category.equals("results")) 
		{
			dbMap.getResults(project, out);
		}
		else if (category.equals("dataprocessing")) 
		{
			dbMap.getDataProcessing(project, out);
		} 
		else if (category.equals("predictions")) 
		{
			dbMap.getPredications(project, out);
		}
	}

	/* Depends on the user's choice, we will take different action to 
	 * display a portion of the project-related data.
	 */
	public void handleOWLGenReport(OntModel theModel, String category, 
		String project, PrintWriter out)
	{       
		if (category.equals("summary")) 
		{
			owlMap.getSummary(theModel, project, out); 
		}
		else if (category.equals("scope")) 
		{
			owlMap.getScope(theModel, project, out);
		}
		else if (category.equals("setupoverview")) 
		{
			owlMap.getSetupOverview(theModel, project, out);
		} 
		else if (category.equals("infrastructure")) 
		{
			owlMap.getInfrastructure(theModel, project, out);
		}
		else if (category.equals("specimen")) 
		{
			owlMap.getSpecimen(theModel, project, out);
		} 
		else if (category.equals("loading")) 
		{
			owlMap.getLoading(theModel, project, out);
		} 
		else if (category.equals("instrumentation")) 
		{
			owlMap.getInstrumentation(theModel, project, out);
		} 
		else if (category.equals("schedule")) 
		{
			owlMap.getSchedule(theModel, project, out);
		}
		else if (category.equals("results")) 
		{
			owlMap.getResults(theModel, project, out);
		}
		else if (category.equals("dataprocessing")) 
		{
			owlMap.getDataProcessing(theModel, project, out);
		} 
		else if (category.equals("predictions")) 
		{
			owlMap.getPredications(theModel, project, out);
		}
	}

	public void genMainPage(PrintWriter out, String theProject, String theChoice, String dType, String owlObj)
	{
		genBackButton(out);

		out.print("<table border=\"0\">\n" + 
			"<tr>\n" + 
			"<td width=180 valign=top>\n" +
			"<form method=\"GET\">\n" +
			"<input type=submit name=\"b_choice\" value=\"Summary\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"Scope\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"SetupOverview\"><p>\n" +	   
			"<input type=submit name=\"b_choice\" value=\"Infrastructure\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"Specimen\"><p>\n" +
   	   		"<input type=submit name=\"b_choice\" value=\"Loading\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"Instrumentation\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"Schedule\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"Results\"><p>\n" +
  	    	"<input type=submit name=\"b_choice\" value=\"DataProcessing\"><p>\n" +
			"<input type=submit name=\"b_choice\" value=\"Predictions\"><p>\n" +
			"</form></td>\n");

		out.print("<td valign=\"top\">");
		if (owlObj.equals("")) 
		{
			dealChoice(out, theProject, theChoice, dType);	
		} 
		else 
		{
			displayObj(out, owlObj);
		}
		out.print("</td></tr></table>");
	}

	private void displayObj(PrintWriter out, String owlObj)
	{
		OWLAccess myAcc = owlMap.getOWLAccess();
		OntModel theModel = owlMap.getOntModel();
		myAcc.displayAnInstance(theModel, owlObj, out);
	}



	public void doGet(HttpServletRequest request,
		HttpServletResponse response)
		throws IOException, ServletException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();

		printHeading(out, "Data Retrieval for Engineering Report Generation:");

		if (request.getParameter("neesobject") != null) 
		{
			String theProject = getSessionAtt(request, "THEPROJECT");
			String owlObj = request.getParameter("neesobject");
			genMainPage(out, theProject, "Summary", "OWL", owlObj);		 
		}
		else if (request.getParameter("OWLChoice") != null)
		{
			String theProject = request.getParameter("THEPROJECT");
			setSessionAtt(request, "THEPROJECT", theProject);
			setSessionAtt(request, "DATATYPE", "OWL");
			genMainPage(out, theProject, "Summary", "OWL", "");
		}
		else if (request.getParameter("DBChoice") != null) 
		{
			String theProject = request.getParameter("THEPROJECT");
			setSessionAtt(request, "THEPROJECT", theProject);
			setSessionAtt(request, "DATATYPE", "DB");
			genMainPage(out, theProject, "Summary", "DB", "");

		}
		else if (request.getParameter("b_back") != null) 
		{
			genFirstPage(out);
		}
		else if	(request.getParameter("b_choice") != null) 
		{
			String theProject = getSessionAtt(request, "THEPROJECT");
			String dType = getSessionAtt(request, "DATATYPE");
			genMainPage(out, theProject, request.getParameter("b_choice"), dType, "");
		}
		else
		{
			genFirstPage(out);
		}

		out.println("</body></html>");
	}

	public void doPost(HttpServletRequest request,
		HttpServletResponse response)
		throws IOException, ServletException
	{
		doGet(request, response);
	}

}
	